// slideshow script for Share.
// Copyright (c) 2007. Grayson Hansard <info@fromconcentratesoftware.com>
// Non-commercial use of this script is permitted provided the above copyright notice is preserved.
// Commercial use may be negotiated with Mr. Hansard.

function slideshow (imageInfoArray) {
	var self = this;
	var currentImageIndex = 0;
	var loadedImages = new Array();
	var timer = null;
	var duration = 2000; //Object.prototype.fadeout && Object.prototype.fadein ? 4000 : 2000;
	this.imageInfoArray = imageInfoArray;
	
	function init () {
		// Begin pre-loading first few images
		var limit = self.imageInfoArray.length > 3 ? 3 : self.imageInfoArray.length;
		for (var i=0; i < limit; i++) {
			SSPreloadImageAtIndex(i);
		};
		
		// Add super element
		var slideshow = document.createElement("div");
		document.body.appendChild(slideshow);
		slideshow.setAttribute("id", "slideshow");
		slideshow.style.display = "none";
		document.body.appendChild(slideshow);
		
		// Add background display
		var bg = document.createElement("div");
		bg.setAttribute("id", "slideshowbg");
		bg.style.position = "absolute";
		bg.style.top = "0";
		bg.style.right = "0";
		bg.style.left = "0";
		bg.style.bottom = "0";
		bg.style.background = "black";
		bg.style.opacity = "0.7";
		bg.style.height = "100%";
		bg.style.width = "100%";
		bg.style.zIndex = "90";
		slideshow.appendChild(bg);
		
		// Add picture content area
		var content = document.createElement("div");
		slideshow.appendChild(content);
		content.setAttribute("id", "slideshowpicture");
		content.style.zIndex = "100";
		content.style.position = "absolute";
		content.style.width = "100%";
		content.style.textAlign = "center";
		content.style.right = "0";
		content.style.bottom = "0";
		content.style.top = "0";
		content.style.marginTop = "50%";
		
		// Add controls
		var controls = document.createElement("div");
		slideshow.appendChild(controls);
		controls.setAttribute("id", "slideshowcontrols");
		controls.style.display = "none";
		controls.style.position = "absolute";
		controls.style.opacity = "1.0";
		controls.style.top = "50%";
		controls.style.textAlign = "center";
		controls.style.width = "100%";
		controls.style.zIndex = "101";
		controls.style.fontSize = "2em";
		controls.style.left = "0";
		
		// Add semi-transparent background
		var cbg = document.createElement("div");
		cbg.style.width = "100%";
		cbg.style.background = "#000";
		cbg.style.opacity = "0.7";
		cbg.innerHTML = "&nbsp;";
		controls.appendChild(cbg);
		
		// Add previous button
		var prev = document.createElement("a");
		prev.id = "slideshowprev";
		prev.href = "#prev";
		prev.style.color = "#fff";
		prev.style.textDecoration = "none";
		prev.style.position = "relative";
		prev.style.top = "-1.1em";
		prev.style.marginRight = "1em";
		prev.style.display = 'none';
		prev.accessKey = 'p';
		prev.addEventListener("click", self.previous, false);
		prev.appendChild(document.createTextNode(String.fromCharCode(8592)));
		controls.appendChild(prev);
		
		// Add playpause button
		var stop = document.createElement("a");
		stop.href = "#stop";
		stop.id = 'slideshowplaypause';
		stop.style.marginRight = "1em";
		stop.style.color = "#fff";
		stop.style.textDecoration = "none";
		stop.style.position = 'relative';
		stop.style.top = '-1.1em';
		stop.addEventListener('click', self.playpause, false);
		stop.appendChild(document.createTextNode(String.fromCharCode(9724)));
		controls.appendChild(stop);
		
		// Add next button
		var next = document.createElement("a");
		next.id = "slideshownext";
		next.href = "#next";
		next.style.color = "#fff";
		next.style.textDecoration = "none";
		next.style.position = "relative";
		next.style.top = "-1.1em";
		next.accessKey = "n";
		next.addEventListener("click", self.next, false);
		next.appendChild(document.createTextNode(String.fromCharCode(8594)));
		controls.appendChild(next);
		
		// Add a close button
		window.addEventListener('keypress', function(e) { if (e.keyCode == 27) self.close(); }, false);
		var close = document.createElement("a");
		slideshow.appendChild(close);
		close.setAttribute("id", "slideshowclose");
		close.href = "#close";
		close.addEventListener('click', self.close, false);
		close.appendChild(document.createTextNode(String.fromCharCode(8855)));
		close.style.position = "absolute";
		close.style.top = "1em";
		close.style.left = "1em";
		close.style.fontSize = "2em";
		close.style.zIndex = "102";
		close.style.textDecoration = "none";
		close.style.color = "#fff";
}
	
	function SSPreloadImageAtIndex(idx) {
		if (idx < 0 || idx > self.imageInfoArray.length) return false;
		var img = new Image();
		var info = self.imageInfoArray[idx]
		img.src = info['path'];
		img.style.position = "relative";
		img.style.marginTop = "-25%";
		var x = '';
		img.style.maxWidth = "640px";
		img.style.maxHeight = "480px";
		img.style.border = "1em solid #fff";
		img.style.borderBottom = "3em solid #fff";
		img.addEventListener('mouseover', showSSControls, false);
		img.addEventListener('mouseout', hideSSControls, false);
		loadedImages.push(img)

		return true;
	}
	
	function showSSControls (e) {
		var controls = document.getElementById("slideshowcontrols");
		controls.style.display = "block";
	}
	
	function hideSSControls (e) {
		if (e.toElement) // toElement doesn't work in Firefox!
			if (e.toElement.id == 'slideshowcontrols' || e.toElement.parentNode.id == 'slideshowcontrols') return;
		else // This doesn't work
			if (this.id == 'slideshowcontrols' || this.parentNode.id == 'slideshowcontrols') return;
		var controls = document.getElementById("slideshowcontrols");
		controls.style.display = "none";
	}
	
	this.next = function() {
		var content = document.getElementById("slideshowpicture");
		var oldPicture = content.firstChild;
		currentImageIndex++;
		if (currentImageIndex >= loadedImages.length) // If we've gone past the end, we loop to the beginning
			currentImageIndex = 0;
		var newPicture = loadedImages[currentImageIndex];
		content.appendChild(newPicture);
		content.removeChild(oldPicture);
		
		// Show the previous button
		document.getElementById("slideshowprev").style.display = 'inline';
		
		// Preload next image if available
		if (currentImageIndex+2 < imageInfoArray.length && loadedImages.length < imageInfoArray.length) SSPreloadImageAtIndex(currentImageIndex+2);
	}
	
	this.previous = function() {
		var content = document.getElementById("slideshowpicture");
		var oldPicture = content.firstChild;
		currentImageIndex--;
		if (currentImageIndex < 0) // If we've gone past the beginning, we loop to the end
			currentImageIndex = loadedImages.length - 1;
		var newPicture = loadedImages[currentImageIndex];
		
		// Hide this if we haven't loaded all of the images and have come back to the first item
		if (currentImageIndex == 0 && loadedImages.length < imageInfoArray.length)
			document.getElementById("slideshowprev").style.display = 'none';
			
		content.appendChild(newPicture);
		content.removeChild(oldPicture);
	}
	
	this.show = function() {
		document.getElementById("slideshow").style.display = "block";
		var content = document.getElementById("slideshowpicture");
		if (!content.firstChild) content.appendChild(loadedImages[currentImageIndex]);
		window.setTimeout(function() {document.getElementById("slideshowcontrols").style.display = 'none'}, 1); // Hack because controls are always shown for some reason
	}
	
	this.close = function() {
		self.stop();
		document.getElementById("slideshow").style.display = "none";
	}
	
	this.start = function() {
		self.show();
		timer = window.setInterval(self.next, duration);
		
		var button = document.getElementById("slideshowplaypause");
		button.firstChild.nodeValue = String.fromCharCode(9724);
		button.parentNode.style.display = 'none';
		button.parentNode.style.display = 'inline';
	}
	
	this.stop = function() {
		window.clearInterval(timer);
		timer = null;
				
		var button = document.getElementById("slideshowplaypause");
		button.firstChild.nodeValue = String.fromCharCode(9656);
		button.parentNode.style.display = 'none';
		button.parentNode.style.display = 'inline';
	}
	
	this.playpause = function() {
		if (timer) self.stop();
		else self.start();
	}
	
	init();
}